"""API for unplash.

https://unsplash.com/developers
"""

import json
import objectpath
import requests

from datahunters.stock.common import StockContent, StockImageAPIBase


class UnsplashImage(StockContent):
  collection = ""
  likes = 0

  def __str__(self):
    return "unsplash image: {}".format(self.full_url)


class UnsplashAPI(StockImageAPIBase):
  api_key = None
  base_url = "https://api.unsplash.com"

  def __init__(self, api_key):
    self.api_key = api_key

  def convert_raw_photo(self, raw_photo):
    """Convert raw photo json to stockimage object.
    """
    stock_img = UnsplashImage()
    stock_img.img_id = raw_photo["id"]
    stock_img.created_date = raw_photo["created_at"]
    stock_img.description = raw_photo["description"]
    stock_img.full_url = objectpath.Tree(raw_photo).execute("$.urls.full")
    stock_img.normal_url = objectpath.Tree(raw_photo).execute("$.urls.regular")
    stock_img.link = objectpath.Tree(raw_photo).execute("$.links.html")
    stock_img.owner = objectpath.Tree(raw_photo).execute("$.user.username")
    stock_img.likes = raw_photo["likes"]
    return stock_img

  def parse_img_res_headers(self, headers):
    """Extract info from response headers.
    """
    total_page_num = None
    next_page_link = None
    remain_limits = headers["X-Ratelimit-Remaining"]
    links = headers["Link"]
    link_parts = links.split(",")
    for link_part in link_parts:
      if link_part.find("last") != -1:
        page_info = link_part.split("&")[-2]
        total_page_num = int(page_info.split("=")[-1])
      if link_part.find("next") != -1:
        next_page_link = link_part.split(";")[0][1:-1]
    next_page_link = next_page_link.strip("<")
    next_page_link = next_page_link.strip(">")
    return total_page_num, next_page_link, remain_limits

  def get_imgs(self, start_page=-1):
    print("start fetching images...")
    if start_page == -1:
      start_page = 1
    next_page_link = "{}/photos?client_id={}&page={}&per_page=30".format(
        self.base_url, self.api_key, start_page)
    while next_page_link:
      try:
        res = requests.get(next_page_link)
        res_json = res.json()
        for raw_photo in res_json:
          cur_img_obj = self.convert_raw_photo(raw_photo)
          yield cur_img_obj
        # get next page link.
        _, next_page_link, remain_limits = self.parse_img_res_headers(
            res.headers)
        if remain_limits == 0:
          print("request limits exceeded.")
          break
      except Exception as ex:
        print("error processing get_imgs request {}: {}".format(
            next_page_link, ex))

  def get_random_imgs(self):
    """Fetch random images.
    """
    next_link = "{}/photos/random?client_id={}&count=30".format(
        self.base_url, self.api_key)
    while True:
      res = requests.get(next_link)
      res_json = res.json()
      for raw_photo in res_json:
        cur_img_obj = self.convert_raw_photo(raw_photo)
        yield cur_img_obj

  def search_imgs(self, keywords):
    next_page_link = "{}/search/photos?client_id={}&query={}&page=1&per_page=30".format(
        self.base_url, self.api_key, keywords)
    while next_page_link:
      res = requests.get(next_page_link)
      # get images.
      res_json = res.json()
      for raw_photo in res_json["results"]:
        cur_img_obj = self.convert_raw_photo(raw_photo)
        yield cur_img_obj
      # get link to next page.
      _, next_page_link = self.parse_img_res_headers(res.headers)


class UnsplashAPITest(object):
  """Class for testing api.
  """
  engine = None

  def __init__(self):
    with open("configs.json", "r") as f:
      configs = json.load(f)
    self.engine = UnsplashAPI(configs["api_key"])

  def test_get_imgs(self):
    img_gen = self.engine.get_imgs()
    print("fetching images...")
    for _ in range(100):
      cur_img = next(img_gen)
      print(cur_img)

  def test_search_imgs(self):
    img_gen = self.engine.search_imgs("people")
    print("searching images...")
    for i in range(10):
      cur_img = next(img_gen)
      print(cur_img)

  def test_random_imgs(self):
    img_gen = self.engine.get_random_imgs()
    for i in range(10):
      cur_img = next(img_gen)
      print(cur_img.full_url)


if __name__ == "__main__":
  api_tester = UnsplashAPITest()
  api_tester.test_get_imgs()



"""Image crawler for ShopStyle api.
"""

import base64
import datetime
import json
import requests
import time
import urllib
import urllib2

import multiprocessing
from tqdm import tqdm

from owl.data import common
from owl.data import img_tools
from owl.data import mongo_manager

# create logger.
logger_name = "shopstyle_local_crawler_logger"
logger = common.get_logger(logger_name, logger_name + ".log")


class ShopStyleParams(object):
  """Parameters for shopstyle api.
  """
  api_name = "ShopStyle"
  base_url = "http://api.shopstyle.com/"
  product_url = "api/v2/products"
  pid = "uid5625-32673475-18"
  format = "json"
  # keywords
  fts = ""
  cat = "category"
  offset = "0"
  page_limit = "50"
  retailer_id = ""
  results_num = 100000
  img_save_dir = ""
  meta_save_dir = ""
  source_name = ""
  # db or file.
  save_type = "db"

  def get_request_url(self):
    """Form url for request.
    """
    return self.base_url + self.product_url + '?pid=' + self.pid + \
        '&cat=' + self.cat + '&format=' + self.format + \
        '&offset=' + self.offset + '&limit=' + self.page_limit + "&fl=" + self.retailer_id


class ShopStyleCrawlerLocal(object):
  """Crawler for shopstyle.

  Metadata is saved to mongodb.
  """
  db_name = "eloquii"  #"eyestyle_data"
  collection_name = "products"
  db_man = None
  index_exists = False

  def init(self):
    """Initialization.
    """
    try:
      # database.
      self.db_man = mongo_manager.MongoManager()
      self.db_man.connect(
          db_name=self.db_name, collection_name=self.collection_name)
      if self.db_man.collection.count() > 0:
        # check if index already exists.
        index_info = self.db_man.collection.index_information()
        if "item_id_1" in index_info:
          self.index_exists = True
          print("item_id index exists")
    except Exception as ex:
      logger.error("init error: {}".format(ex.message))
      raise

  def save_item_to_db(self, item_id, img_url, item_metadata, sparams):
    """Save item to mongodb.

    Args:
      item_id: item id.
      img_url: product img url.
      sparams: parameters.
      item_metadata: metadata for the item.
    """
    try:
      user_agent = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7"
      header = {'User-agent': user_agent}

      # check if already exists.
      res = self.db_man.query(attribute="item_id", value_list=[item_id])
      if len(res) > 0:
        return

      # req_data = requests.get(product_img_url, headers=header)
      startt = time.time()
      req_data = urllib2.Request(img_url, None, header)
      img_bin = urllib2.urlopen(req_data).read()
      img_base64 = base64.b64encode(img_bin)
      print("save image time: {}".format(time.time() - startt))
      now = datetime.datetime.now()
      save_date = "{}-{}-{}".format(now.year, now.month, now.day)
      new_item = {
          "item_id": item_id,
          "original_id": original_id,
          "date": save_date,
          "category": item_metadata["categories"][0]["id"],
          "img_data": img_base64,
          "in_stock": item_metadata["inStock"],
          "raw_metadata": json.dumps(item_metadata)
      }
      # add item to database.
      startt = time.time()
      self.db_man.add(new_item)
      # create index.
      if not self.index_exists:
        self.db_man.collection.create_index("item_id")
        logger.info("item_id index created.")
        self.index_exists = True
      print("save mongodb time: {}s".format(time.time() - startt))
    except Exception as ex:
      logger.error("error in saving to db: {}".format(ex.message))

  def par_retrieve_db(self, items, sparams):
    """Parallel retreive to save to database.

    Form item_id as productid__imgid___datasource

    Args:
      items: product items to save.
      sparams: request parameters.
    """
    to_debug = False

    # create db item.
    for item in tqdm(items):
      try:
        #print('processing {}th product'.format(cnt))
        product_id = str(item["id"])
        use_color = False
        if "colors" in item:
          # save all color versions.
          if to_debug:
            logger.info("{} colors found".format(len(item["colors"])))
          for color_item in item["colors"]:
            if "image" not in color_item:
              continue
            product_img_id = color_item["image"]["id"]
            item_id = "{}__{}___shopstyle".format(product_id, product_img_id)
            product_img_url = color_item["image"]["sizes"]["Original"]["url"]
            self.save_item_to_db(item_id, product_img_url, item, sparams)
            use_color = True

        # just use one image.
        if not use_color:
          if "image" not in item:
            return
          if to_debug:
            logger.info("not using color")
          product_img_id = item["image"]["id"]
          item_id = "{}__{}___shopstyle".format(product_id, product_img_id)
          product_img_url = item["image"]["sizes"]["Original"]["url"]
          self.save_item_to_db(item_id, product_img_url, item, sparams)

      except Exception as e:
        logger.error("error in shopstyle crawler: {}".format(e.message))

  def crawl(self, params):
    """Crawl products from api.

    Args:
      params: params for crawling.
    """
    try:
      # get total number.
      params.offset = "0"
      url = params.get_request_url()
      res_data = requests.get(url)
      res_json = json.loads(res_data.content)
      total_num = int(res_json["metadata"]["total"])
      total_num = min(params.results_num, total_num)
      logger.info("category {} has total {} products.".format(
          params.cat, total_num))
      page_num = max(1, total_num / int(params.page_limit))
      logger.info("total page number to fetch: {}".format(page_num))

      # make request for each page.
      for pid in range(page_num):
        try:
          logger.info("making api call for {} at page {}:{}".format(
              params.cat, pid + 1, page_num))
          cur_offset = int(params.page_limit) * pid
          if cur_offset > total_num:
            break
          params.offset = str(cur_offset)
          url = params.get_request_url()
          # make request.
          res_data = requests.get(url)
          res_json = json.loads(res_data.content)
        except Exception as ex:
          logger.error("error in response: {}".format(ex.message))
          continue

        # parse results.
        if len(res_json["products"]) == 0:
          logger.warning("empty response")
        else:
          self.par_retrieve_db(res_json["products"], params)

        # parse for api call
        time.sleep(3)
    except Exception as ex:
      logger.error("error in crawl: {}".format(ex.message))


def crawl_one_cate_per_process(cate):
  """Crawl one category in a standalone process.

    Args:
      cate: category name.
    """
  try:
    crawler = ShopStyleCrawlerLocal()
    crawler.init()
    sparams = ShopStyleParams()
    sparams.cat = cate
    crawler.crawl(sparams)
  except Exception as ex:
    logger.error("error crawl {}: ".format(cate, ex.messeage))


def run_crawler_parallel():
  """Run crawler in multiple processes.
  """
  try:
    logger.info("parallel crawler started...")
    # get category list.
    cate_fn = "shopstyle_categories.json"
    cate_data = json.load(open(cate_fn, 'r'))
    cates = []
    for gender in cate_data:
      for cate in cate_data[gender]:
        for subcate in cate["sub"]:
          cates.append(subcate["id"])
    # start processing.
    num_cores = multiprocessing.cpu_count()
    pool = multiprocessing.Pool(processes=num_cores / 2)
    pool.map(crawl_one_cate_per_process, cates[-4:])
    logger.info("completed crawl.")
  except Exception as ex:
    logger.error("error in main parallel crawler: {}".format(ex.message))


def run_crawler_single():
  """Run crawler in single process.
  """
  try:
    logger.info("crawler started...")
    crawler = ShopStyleCrawlerLocal()
    crawler.init()
    sparams = ShopStyleParams()
    cate_fn = "shopstyle_categories.json"
    cate_data = json.load(open(cate_fn, 'r'))
    cates = []
    for gender in cate_data:
      for cate in cate_data[gender]:
        for subcate in cate["sub"]:
          cates.append(subcate["id"])
    for cate in cates:
      sparams.cat = cate
      crawler.crawl(sparams)
    logger.info("completed crawl.")
  except Exception as ex:
    logger.error("main exception: {}".format(ex.message))


def run_crawler_retailer(retailer):
  """Run crawler in single process.
  """
  try:
    logger.info("crawler started...")
    crawler = ShopStyleCrawlerLocal()
    crawler.init()
    sparams = ShopStyleParams()
    sparams.retailer_id = retailer
    crawler.crawl(sparams)
    logger.info("completed crawl.")
  except Exception as ex:
    logger.error("main exception: {}".format(ex.message))


if __name__ == "__main__":
  run_crawler_retailer("r2001")
  # run_crawler_parallel()
  # while True:
  #   run_crawler_parallel()
  #   # wait for 10 seconds to start next.
  #   time.sleep(10)



"""Shared definition for search engine api.
"""

import abc
import json


class ImgSearchObject(object):
  """Result object from image search.
  """
  thumbnail_url = None
  img_url = None
  link_url = None

  def to_json(self):
    """Convert to json.
    """
    return json.loads(json.dumps(self.__dict__))


class SearchEngineAPIBase(object):
  """Base class for search engine api.
  """
  __metaclass__ = abc.ABCMeta

  def __init__(self):
    pass

  def text_search(self, keywords):
    pass

  def img_search(self, keywords):
    pass



"""Tinder api.
"""

import json

import pynder

from datahunters.social.common import SocialUser
from datahunters.social.tools import gen_fb_token


class TinderAPI(object):
  """API Class for Tinder.
  """
  # tinder session.
  engine = None

  def __init__(self, fb_email, fb_pw):
    try:
      fb_token = gen_fb_token(fb_email, fb_pw)
      self.engine = pynder.Session(fb_token)
      print("tinder api started")
    except Exception as ex:
      print("fail to create tinder api: {}".format(ex))

  def convert_to_socialuser(self, raw_user):
    """Convert api raw user to socialuser object.
    """
    user = SocialUser()
    user.name = raw_user.name
    user.photos = []
    for photo in raw_user.photos:
      user.photos.append(photo)
    user.age = raw_user.age
    user.gender = raw_user.gender
    user.custom["jobs"] = raw_user.jobs
    # school has problems
    user.custom["schools"] = raw_user.schools
    user.custom["distance_km"] = raw_user.distance_km
    user.custom["instagram"] = raw_user.instagram_username
    return user

  def get_profile(self):
    """Retrieve profile.
    """
    profile = self.engine.profile
    user = SocialUser()
    user.name = profile.name
    user.photos = profile.photos
    if profile.gender == 0:
      user.gender = "male"
    else:
      user.gender = "female"
    return user

  def candidate_generator(self):
    """Get candidate matches.

    Returns: generator of socialuser.
    """
    try:
      raw_users = self.engine.nearby_users()
      for raw_user in raw_users:
        try:
          user = self.convert_to_socialuser(raw_user)
          yield user
        except Exception as ex:
          print("candidate convertion error: {}".format(ex))
          continue
      yield None
    except Exception as ex:
      print("error in fetching candidate: {}".format(ex))
      yield None

  def get_existing_matches(self):
    """Get already matched users.
    """
    raw_matches = self.engine.matches()
    matches = []
    for raw_match in raw_matches:
      user = self.convert_to_socialuser(raw_match.user)
      matches.append(user)
    print("total {} matches found".format(len(matches)))
    return matches


if __name__ == "__main__":
  api = TinderAPI("flyfengjie@gmail.com", "detective3!")
  user_gen = api.candidate_generator()
  # api.get_existing_matches()
  cnt = 0
  while True:
    user = next(user_gen)
    if user is None:
      break
    # print user
    # print type(user)
    cnt += 1
    if cnt == 1:
      break
  # print cnt



"""Google api class.
"""

import urllib

from datahunters.shared.selenium_scraper import SeleniumScraper
from datahunters.search.common import ImgSearchObject


class GoogleImageAPI(SeleniumScraper):
  """Class for google image search api.
  """
  base_url = "https://www.google.com/search"

  def __init__(self, use_headless=True):
    super().__init__(use_headless)
    print("Google image api initialized")

  def scrape(self, keywords):
    try:
      print("start scraping '{}' using Google".format(keywords))
      formatted_keywords = keywords.strip().replace(" ", "+")
      req_url = "{}?q={}&source=lnms&tbm=isch&sa=X&ei=0eZEVbj3IJG5uATalICQAQ&ved=0CAcQ_AUoAQ&biw=939&bih=591".format(
          self.base_url, formatted_keywords)
      elems = self.scrape_inf_scroll(
          req_url, "a.rg_l", "a.rg_l", load_btn_selector="input#smb")
      print("total fetched items: {}".format(len(elems)))
      for elem in elems:
        try:
          # create result object.
          cur_res = ImgSearchObject()
          link = self.get_attribute_values([elem], "href")[0]
          link = urllib.parse.unquote(link)
          cur_res.img_url = link[link.find("imgurl=") + 7:link.find(
              "&imgrefurl")].rstrip(".")
          yield cur_res
        except Exception as ex:
          print("error in processing item: {}".format(ex))
          continue
      self.browser.close()
      print("Google image scraping finished.")
      yield None
    except Exception as ex:
      print("error in Google image scraper: {}".format(ex))
      self.browser.close()
      yield None


if __name__ == "__main__":
  api = GoogleImageAPI()
  img_gen = api.scrape("car crash")
  cnt = 0
  while True:
    res = next(img_gen)
    if res is None:
      break
    cnt += 1
    if cnt % 10 == 0:
      print("scraped {} images".format(cnt))
  print("scrape finished.")



"""Tests for scraper.
"""

import json

from scraper import ShopStyleParams, ShopStyleScraper


class TestScraper(object):
  def __init__(self):
    with open("configs/config.json", "r") as f:
      configs = json.load(f)
    self.api_key = configs["api_key"]

  def test_cat(self):
    params = ShopStyleParams()
    my_scraper = ShopStyleScraper(self.api_key)
    cats = my_scraper.get_categories()
    print cats
    assert True


from datahunters import ecom
from datahunters import search
from datahunters import social
from datahunters import movies
from datahunters import stock
from datahunters import shared
from datahunters import assets


"""Shopstyle scraper.
"""

import datetime
import json
import os
import requests
import time
import urllib

import objectpath


class ShopStyleParams(object):
  """Parameters for shopstyle api.
  """
  api_name = "ShopStyle"
  base_url = "http://api.shopstyle.com/"
  product_url = "api/v2/products"
  pid = "uid5625-32673475-18"
  format = "json"
  # keywords
  fts = ""
  cat = "category"
  offset = "0"
  page_limit = "50"
  results_num = 100000
  retailer_id = ""
  img_save_dir = ""
  meta_save_dir = ""
  source_name = ""

  def get_request_url(self):
    """Form url for request.
    """
    url = self.base_url + self.product_url + '?pid=' + self.pid + \
        '&cat=' + self.cat + '&format=' + self.format + \
        '&offset=' + self.offset + '&limit=' + self.page_limit
    if self.retailer_id != "":
      url += "&fl=" + self.retailer_id
    return url


class ShopStyleScraper(object):
  """Class for scraping shopstyle.
  """
  api_key = ""
  logger = None

  def __init__(self, api_key, logger=None):
    self.api_key = api_key
    self.logger = logger
    print("shopstyle scraper initiated.")

  def get_total_product_num(self, params):
    """Get total number of products under params.
    """
    try:
      params.api_key = self.api_key
      url = params.get_request_url()
      res_data = requests.get(url)
      res_json = json.loads(res_data.content)
      total_num = int(res_json["metadata"]["total"])
      total_num = min(params.results_num, total_num)
      return total_num
    except Exception as ex:
      print("error in getting product num: {}".format(ex.message))

  def get_categories(self):
    """Fetch all categories.
    """
    try:
      cat_url = "http://api.shopstyle.com/api/v2/categories?pid={}".format(
          self.api_key)
      res_data = requests.get(cat_url)
      res_json = json.loads(res_data.content)
      cats = res_json["categories"]
      return cats
    except Exception as ex:
      if self.logger:
        self.logger.error(ex.message)

  def get_brands(self):
    """Fetch all brands.
    """
    try:
      brand_url = "http://api.shopstyle.com/api/v2/brands?pid={}".format(
          self.api_key)
      res_data = requests.get(brand_url)
      res_json = json.loads(res_data.content)
      return res_json["brands"]
    except Exception as ex:
      if self.logger:
        self.logger.error(ex.message)

  def get_retailers(self):
    """Fetch all retailers.
    """
    try:
      retailer_url = "http://api.shopstyle.com/api/v2/retailers?pid={}".format(
          self.api_key)
      res_data = requests.get(retailer_url)
      res_json = json.loads(res_data.content)
      return res_json["retailers"]
    except Exception as ex:
      if self.logger:
        self.logger.error(ex.message)

  def create_item(self, original_id, img_url, item_metadata):
    """Create item with essential info.

    No item_id is specified.
    """
    save_date = datetime.datetime.now().isoformat()
    target_item = {
        "original_id": original_id,
        "data_source": "shopstyle",
        "scrape_date": save_date,
        "name": item_metadata["name"],
        "img_urls": [img_url],
        "product_link": item_metadata["clickUrl"],
        "in_stock": item_metadata["inStock"],
        "price": item_metadata["price"],
        "currency": item_metadata["currency"]
    }
    target_item["brand"] = objectpath.Tree(item_metadata).execute(
        "$.brand.name")
    target_item["retailer"] = objectpath.Tree(item_metadata).execute(
        "$.retailer.name")
    target_item["category"] = objectpath.Tree(item_metadata).execute(
        "$.categories[0].id")
    target_item["last_modified"] = objectpath.Tree(item_metadata).execute(
        "$.lastModified")
    # clean and remove none.
    for key, val in target_item.iteritems():
      if val is None:
        target_item[key] = "unknown"
    return target_item

  def retrieve_items(self, items, sparams):
    """Retrieve items from api response.

    Args:
      items: product items to save.
      sparams: request parameters.
    """
    new_items = []
    for item in items:
      try:
        product_id = str(item["id"])
        use_color = False
        if "color" in item:
          # save all color versions.
          for color_item in item["colors"]:
            if "image" not in color_item:
              continue
            product_img_id = color_item["image"]["id"]
            original_id = "{}__{}".format(product_id, product_img_id)
            product_img_url = color_item["image"]["sizes"]["Original"]["url"]
            new_item = self.create_item(original_id, product_img_url, item)
            new_items.append(new_item)
            use_color = True
        # just use one image.
        if not use_color:
          if "image" not in item:
            return
          product_img_id = item["image"]["id"]
          original_id = "{}__{}".format(product_id, product_img_id)
          product_img_url = item["image"]["sizes"]["Original"]["url"]
          new_item = self.create_item(original_id, product_img_url, item)
          new_items.append(new_item)
      except Exception as e:
        if self.logger:
          self.logger.error("error in shopstyle crawler: {}".format(e.message))

    return new_items

  def scrape_products(self, params):
    """Start scraping.

    Returns:
      scraped items for a page determined by offset.
    """
    all_items = []
    try:
      print("making api call for {} at page offset {}".format(
          params.cat, params.offset))
      url = params.get_request_url()
      res_data = requests.get(url)
      res_json = json.loads(res_data.content)
      # parse results.
      if len(res_json["products"]) == 0:
        print("empty response")
      else:
        items = self.retrieve_items(res_json["products"], params)
        all_items.extend(items)
    except Exception as ex:
      print(ex)

    return all_items


if __name__ == "__main__":
  with open("configs/config.json", "r") as f:
    configs = json.load(f)
  scraper = ShopStyleScraper(configs["api_key"])
  params = ShopStyleParams()
  print(scraper.get_brands())



"""For distribution.
"""

from setuptools import setup, find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from subprocess import check_call


class PostDevelopCommand(develop):
  """Post-installation for development mode."""

  def run(self):
    check_call("apt-get install this-package".split())
    develop.run(self)


class PostInstallCommand(install):
  """Post-installation for installation mode."""

  def run(self):
    # PUT YOUR POST-INSTALL SCRIPT HERE or CALL A FUNCTION
    install.run(self)


with open("./configs/requirements.txt", "r") as f:
  dep_packages = f.readlines()
  # remove local install.
  dep_packages = [x.strip() for x in dep_packages if not x.startswith("-e")]
  # print(dep_packages)

setup(
    name="datahunters",
    version="0.0.3",
    description="library for collecting data",
    url="https://github.com/perceptance/datahunters",
    author="Jie Feng",
    author_email="jiefengdev@gmail.com",
    license="MIT",
    include_package_data=True,
    packages=find_packages("./"),
    install_requires=dep_packages)
